# Copyright 2021 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

import base64
import json

from odoo import api, models
from odoo import fields as odooFields
from odoo.exceptions import UserError


class AccountInvoice(models.Model):
    _name = 'account.invoice'
    _inherit = 'account.invoice'

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    project_id = odooFields.Many2one(
        'project.project',
        string='Projet',
        change_default=True,
        readonly=True,
        states={'draft': [('readonly', False)]},
        track_visibility='always',
        ondelete='restrict',)
    num_dossier = odooFields.Char(related="project_id.num_dossier")
    type_convention_id = odooFields.Many2one(
        "adefpat.type.convention",
        related="project_id.type_convention_id",
        string="Type de convention de financement",
    )
    alfresco_file = odooFields.Char("Facture liée")
    task_ids = odooFields.One2many(
        "project.task",
        "invoice_id",
        string="Séances liées",
    )
    partner_id_domain = odooFields.Char(
        compute="_compute_partner_id_domain",
        readonly=True,
        store=False,
    )
    task_ids_domain = odooFields.Char(
        compute="_compute_task_ids_domain",
        readonly=True,
        store=False,
    )
    amount_task_total = odooFields.Monetary(
        string='Total Séances', store=True,
        readonly=True, compute='_compute_amount_task_total')

    @api.one
    @api.depends('task_ids', 'task_ids.cout_seance')
    def _compute_amount_task_total(self):
        if self.type == "in_invoice":
            self.amount_task_total = sum(
                line.cout_seance for line in self.task_ids)

    @api.depends('project_id')
    def _compute_partner_id_domain(self):
        """
        Crée un domaine dynamique pour les CF
        """
        if self.type == "in_invoice":
            domain = [('is_consultant_form', '=', True)]
            if self.project_id:
                formateur_ids = self.env["project.task"].search(
                    [("project_id", "=", self.project_id.id)]
                ).mapped("formateur_id")
                domain.append(('id', 'in', formateur_ids.ids))
            self.partner_id_domain = json.dumps(domain)

    @api.depends('project_id', 'partner_id')
    def _compute_task_ids_domain(self):
        """
        Crée un domaine dynamique pour les séances
        """
        if self.type == "in_invoice":
            domain = []
            dom = []
            if self.project_id:
                dom.append(("project_id", "=", self.project_id.id))
            if self.partner_id:
                dom.append(("formateur_id", "=", self.partner_id.id))
            task_ids = self.env["project.task"].search(dom)
            domain.append(('id', 'in', task_ids.ids))
            self.task_ids_domain = json.dumps(domain)

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------
    @api.model
    def _prepare_refund(self, invoice, date_invoice=None, date=None, description=None, journal_id=None):
        values = super(AccountInvoice, self)._prepare_refund(invoice, date_invoice, date, description, journal_id)
        if invoice.project_id:
            values['project_id'] = invoice.project_id.id
        return values

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
